'use strict';
angular.module('ui.dashboard', ['ui.bootstrap', 'ui.sortable']);

angular.module('ui.dashboard').directive('dashboard', ['WidgetModel', 'WidgetDefCollection', '$modal', 'DashboardState', '$log',
function (WidgetModel, WidgetDefCollection, $modal, DashboardState, $log){
    return{
      restrict: 'A',
      templateUrl: function(element, attr){
        return attr.templateUrl ? attr.templateUrl : 'template/dashboard.html';
      },
      scope: true,
      controller: ['$scope', '$attrs', function (scope, attrs){
        // default options
        var defaults={
          stringifyStorage: true,
          hideWidgetSettings: false,
          hideWidgetClose: true
        }; 

        scope.options=scope.$eval(attrs.dashboard);

        // Deep options
        scope.options.settingsModalOptions=scope.options.settingsModalOptions ||{};
        _.each(['settingsModalOptions'], function(key){
          // Ensure it exists on scope.options
          scope.options[key]=scope.options[key] ||{};
          // Set defaults
          _.defaults(scope.options[key], defaults[key]);
        });

        // Shallow options
        _.defaults(scope.options, defaults);

        var sortableDefaults={
          stop: function (){
            scope.saveDashboard();
          },
          handle: '.widget-header'
        };
        scope.sortableOptions=angular.extend({}, sortableDefaults, scope.options.sortableOptions ||{});
      }],
      link: function (scope){

          // Save default widget config for reset
        scope.defaultWidgets=scope.options.defaultWidgets;

        //scope.widgetDefs=scope.options.widgetDefinitions;
        scope.widgetDefs=new WidgetDefCollection(scope.options.widgetDefinitions);
        var count=1;

        // Instantiate new instance of dashboard state
        scope.dashboardState=new DashboardState(
          scope.options.storage,
          scope.options.storageId,
          scope.options.storageHash,
          scope.widgetDefs,
          scope.options.stringifyStorage
        );

        /**
         * Instantiates a new widget on the dashboard
         * @param{Object} widgetToInstantiate The definition object of the widget to be instantiated
         */
        scope.addWidget=function (widgetToInstantiate, doNotSave){
          var defaultWidgetDefinition=scope.widgetDefs.getByName(widgetToInstantiate.name);
          if (!defaultWidgetDefinition){
            throw 'Widget ' + widgetToInstantiate.name + ' is not found.';
          }

          // Determine the title for the new widget
          var title;
          if (widgetToInstantiate.title){
            title=widgetToInstantiate.title;
          } else if (defaultWidgetDefinition.title){
            title=defaultWidgetDefinition.title;
          } else{
            title='Widget ' + count++;
          }

          // Deep extend a new object for instantiation
          widgetToInstantiate=jQuery.extend(true,{}, defaultWidgetDefinition, widgetToInstantiate);

          // Instantiation
          var widget=new WidgetModel(widgetToInstantiate,{
            title: title
          });

          scope.widgets.push(widget);
          if (!doNotSave){
            scope.saveDashboard();
          }

          return widget;
        };

        /**
         * Removes a widget instance from the dashboard
         */
        scope.removeWidget=function (widget){
            var arrLength=(scope.widgets).length;

            for (var i=0; i < arrLength; i++){
                if (scope.widgets[i].name===widget.name){
                    scope.widgets.splice(i,1);
                    scope.saveDashboard();

                    // firing an event upwards
                    scope.$emit('removeWidgetEvent',widget);
                    break;
                }
           }
        };

        /**
         * Opens a dialog for setting and changing widget properties
         */
        scope.openWidgetSettings=function (widget ,index){
           scope.$broadcast('dialogopenEvent',widget,index);
        };
		
		scope.openManualRefreshSettings = function (widget,index){
           scope.$broadcast('manualRefreshEvent',widget,index);
        };

        /**
         * Remove   all widget instances from dashboard
         */
        scope.clear=function (doNotSave){
          scope.widgets=[];
          if (doNotSave===true){
            return;
          }
          scope.saveDashboard();
        };

        /**
         * Used for preventing default on click event
         * @param{Object} event     A click event
         * @param{Object} widgetDef A widget definition object
         */
        scope.addWidgetInternal=function (event, widgetDef){
          event.preventDefault();
          scope.addWidget(widgetDef);
        };

        /**
         * Uses dashboardState service to save state
         */
        scope.saveDashboard=function (force){
          if (!scope.options.explicitSave){
            scope.dashboardState.save(scope.widgets);
          } else{
            if (!angular.isNumber(scope.options.unsavedChangeCount)){
              scope.options.unsavedChangeCount=0;
            }
            if (force){
              scope.options.unsavedChangeCount=0;
              scope.dashboardState.save(scope.widgets);

            } else{
              ++scope.options.unsavedChangeCount;
            }
          }
        };

        /**
         * Wraps saveDashboard for external use.
         */
        scope.externalSaveDashboard=function(){
          scope.saveDashboard(true);
        };

        /**
         * Clears current dash and instantiates widget definitions
         * @param{Array} widgets Array of definition objects
         */
        scope.loadWidgets=function (widgets){
          // AW dashboards are continuously saved today (no "save" button).
          scope.savedWidgetDefs=widgets;
          scope.clear(true);
          _.each(widgets, function (widgetDef){
            scope.addWidget(widgetDef, true);
          });
        };
          
         /**
         * Resets widget instances to default config
         * @return {[type]} [description]
         */
        scope.resetWidgetsToDefault = function () {
          scope.loadWidgets(scope.defaultWidgets);
          scope.saveDashboard();
        };

        // Set default widgets array
        var savedWidgetDefs=scope.dashboardState.load();

        // Success handler
        function handleStateLoad(saved){
          scope.options.unsavedChangeCount=0;
          if (saved && saved.length){
            scope.loadWidgets(saved);
          } else if (scope.defaultWidgets){
            scope.loadWidgets(scope.defaultWidgets);
          } else{
            scope.clear(true);
          }
        }

        if (angular.isArray(savedWidgetDefs)){
          handleStateLoad(savedWidgetDefs);
        } else if (savedWidgetDefs && angular.isObject(savedWidgetDefs) && angular.isFunction(savedWidgetDefs.then)){
          savedWidgetDefs.then(handleStateLoad, handleStateLoad);
        } else{
          handleStateLoad();
        }

        // expose functionality externally
        // functions are appended to the provided dashboard options
        scope.options.addWidget=scope.addWidget;
        scope.options.loadWidgets=scope.loadWidgets;
        scope.options.saveDashboard=scope.externalSaveDashboard;
        scope.options.removeWidget=scope.removeWidget;
        scope.options.openWidgetSettings=scope.openWidgetSettings;

        // save state
        scope.$on('widgetChanged', function (event){
          event.stopPropagation();
          scope.saveDashboard();
        });
      }
    };
}]);

/*
 * Define widget directive.
 */
'use strict';
angular.module('ui.dashboard').directive('widget', ['$injector', function ($injector){
    return{
      controller: 'DashboardWidgetCtrl',
      link: function (scope){
        var widget=scope.widget;
        var dataModelType=widget.dataModelType;

        // set up data source
        if (dataModelType){
          var DataModelConstructor; // data model constructor function

          if (angular.isFunction(dataModelType)){
            DataModelConstructor=dataModelType;
          } else if (angular.isString(dataModelType)){
            $injector.invoke([dataModelType, function (DataModelType){
              DataModelConstructor=DataModelType;
            }]);
          } else{
            throw new Error('widget dataModelType should be function or string');
          }

          var ds;
          if (widget.dataModelArgs){
            ds=new DataModelConstructor(widget.dataModelArgs);
          } else{
            ds=new DataModelConstructor();
          }
          widget.dataModel=ds;
          ds.setup(widget, scope);
          ds.init();
          scope.$on('$destroy', _.bind(ds.destroy,ds));
        }

        // Compile the widget template, emit add event
        scope.compileTemplate();
        scope.$emit('widgetAdded', widget);
      }
    };
}]);

/*
 * Define layout storage factory.
 */
'use strict';
angular.module('ui.dashboard').factory('LayoutStorage', function(){
    var noopStorage={

      setItem: function(){
      },
      getItem: function(){
      },
      removeItem: function(){
      }
    };

    function LayoutStorage(options){
      var defaults={
        storage: noopStorage,
        storageHash: '',
        stringifyStorage: true
      };

      angular.extend(defaults, options);
      angular.extend(options, defaults);

      this.id=options.storageId;
      this.storage=options.storage;
      this.storageHash=options.storageHash;
      this.stringifyStorage=options.stringifyStorage;
      this.widgetDefinitions=options.widgetDefinitions;
      this.defaultLayouts=options.defaultLayouts;
      this.lockDefaultLayouts=options.lockDefaultLayouts;
      this.widgetButtons=options.widgetButtons;
      this.explicitSave= options.explicitSave;
      this.defaultWidgets=options.defaultWidgets;
      this.options=options;
      this.options.unsavedChangeCount=0;
      this.layouts=[];
      this.states={};
      this.load();
      this._ensureActiveLayout();
    }

    LayoutStorage.prototype={

      add: function(layouts){
        if (!angular.isArray(layouts)){
          layouts=[layouts];
        }
        var self=this;
        angular.forEach(layouts, function(layout){
          layout.dashboard=layout.dashboard ||{};
          layout.dashboard.storage=self;
          layout.dashboard.storageId=layout.id=self._getLayoutId.call(self,layout);
          layout.dashboard.widgetDefinitions=self.widgetDefinitions;
          layout.dashboard.stringifyStorage=false;
          layout.dashboard.defaultWidgets=layout.defaultWidgets || self.defaultWidgets;
          layout.dashboard.widgetButtons=self.widgetButtons;
          layout.dashboard.explicitSave= self.explicitSave;
          self.layouts.push(layout);
        });
      },

      remove: function(layout){
        var index=this.layouts.indexOf(layout);
        if (index >=0){
          this.layouts.splice(index, 1);
          delete this.states[layout.id];

          // check for active
          if (layout.active && this.layouts.length){
            var nextActive=index > 0 ? index - 1 : 0;
            this.layouts[nextActive].active=true;
          }
        }
      },

      save: function(){
        var state={
          layouts: this._serializeLayouts(),
          states: this.states,
          storageHash: this.storageHash
        };

        if (this.stringifyStorage){
          state=JSON.stringify(state);
        }

        this.storage.setItem(this.id, state);
        this.options.unsavedChangeCount=0;
      },

      load: function(){
        var serialized=this.storage.getItem(this.id);

        this.clear();

        if (serialized){
          // check for promise
          if (angular.isObject(serialized) && angular.isFunction(serialized.then)){
            this._handleAsyncLoad(serialized);
          } else{
            this._handleSyncLoad(serialized);
          }
        } else{
          this._addDefaultLayouts();
        }
      },

      clear: function(){
        this.layouts=[];
        this.states={};
      },

      setItem: function(id, value){
        this.states[id]=value;
        this.save();
      },

      getItem: function(id){
        return this.states[id];
      },

      removeItem: function(id){
        delete this.states[id];
        this.save();
      },

      getActiveLayout: function(){
        var len=this.layouts.length;
        for (var i=0; i < len; i++){
          var layout=this.layouts[i];
          if (layout.active){
            return layout;
          }
        }
        return false;
      },

      _addDefaultLayouts: function(){
        var self=this;
        var defaults=this.lockDefaultLayouts ?{ locked: true } :{};
        angular.forEach(this.defaultLayouts, function(layout){
          self.add(angular.extend(_.clone(defaults), layout));
        });
      },

      _serializeLayouts: function(){
        var result=[];
        angular.forEach(this.layouts, function(l){
          result.push({
            title: l.title,
            id: l.id,
            active: l.active,
            locked: l.locked,
            defaultWidgets: l.dashboard.defaultWidgets
          });
        });
        return result;
      },

      _handleSyncLoad: function(serialized){

        var deserialized;

        if (this.stringifyStorage){
          try{
            deserialized=JSON.parse(serialized);
          } catch (e){
            this._addDefaultLayouts();
            return;
          }
        } else{
          deserialized=serialized;
        }

        if (this.storageHash !==deserialized.storageHash){
          this._addDefaultLayouts();
          return;
        }
        this.states=deserialized.states;
        this.add(deserialized.layouts);
      },

      _handleAsyncLoad: function(promise){
        var self=this;
        promise.then(
          angular.bind(self, this._handleSyncLoad),
          angular.bind(self, this._addDefaultLayouts)
        );
      },

      _ensureActiveLayout: function(){
        for (var i=0; i < this.layouts.length; i++){
          var layout=this.layouts[i];
          if (layout.active){
            return;
          }
        }
        if (this.layouts[0]){
          this.layouts[0].active=true;
        }
      },

      _getLayoutId: function(layout){
        if (layout.id){
          return layout.id;
        }
        var max=0;
        for (var i=0; i < this.layouts.length; i++){
          var id=this.layouts[i].id;
          max=Math.max(max, id * 1);
        }
        return max + 1;
      }

    };
    return LayoutStorage;
});

/*
 * Define dashboard state factory.
 */
'use strict';
angular.module('ui.dashboard')
  .factory('DashboardState', ['$log', '$q', function ($log, $q){
    function DashboardState(storage, id, hash, widgetDefinitions, stringify){
      this.storage=storage;
      this.id=id;
      this.hash=hash;
      this.widgetDefinitions=widgetDefinitions;
      this.stringify=stringify;
    }

    DashboardState.prototype={
      /**
       * Takes array of widget instance objects, serializes, 
       * and saves state.
       * 
       * @param {Array} widgets  scope.widgets from dashboard directive
       * @return{Boolean}        true on success, false on failure
       */
      save: function (widgets){
        
        if (!this.storage){
          return true;
        }

        var serialized=_.map(widgets, function (widget){
          var widgetObject={
            title: widget.title,
            name: widget.name,
            style: widget.style,
            size: widget.size,
            dataModelOptions: widget.dataModelOptions,
            storageHash: widget.storageHash,
            attrs: widget.attrs
          };

          return widgetObject;
        });

        var item={ widgets: serialized, hash: this.hash };

        if (this.stringify){
          item=JSON.stringify(item);
        }

        this.storage.setItem(this.id, item);
        return true;
      },

      /**
       * Loads dashboard state from the storage object.
       * Can handle a synchronous response or a promise.
       * 
       * @return{Array|Promise} Array of widget definitions or a promise
       */
      load: function (){
        if (!this.storage){
          return null;
        }

        var serialized;

        // try loading storage item
        serialized=this.storage.getItem( this.id );

        if (serialized){
          // check for promise
          if (angular.isObject(serialized) && angular.isFunction(serialized.then)){
            return this._handleAsyncLoad(serialized);
          }
          // otherwise handle synchronous load
          return this._handleSyncLoad(serialized);
        } else{
          return null;
        }
      },

      _handleSyncLoad: function(serialized){

        var deserialized, result=[];

        if (!serialized){
          return null;
        }

        if (this.stringify){
          try{ // to deserialize the string
            deserialized=JSON.parse(serialized);

          } catch (e){
            // bad JSON, log a warning and return
            $log.warn('Serialized dashboard state was malformed and could not be parsed: ', serialized);
            return null;
          }
        }
        else{
          deserialized=serialized;
        }

        // check hash against current hash
        if (deserialized.hash !==this.hash){
          $log.info('Serialized dashboard from storage was stale (old hash: ' + deserialized.hash + ', new hash: ' + this.hash + ')');
          this.storage.removeItem(this.id);
          return null;
        }

        // Cache widgets
        var savedWidgetDefs=deserialized.widgets;

        // instantiate widgets from stored data
        for (var i=0; i < savedWidgetDefs.length; i++){

          // deserialized object
          var savedWidgetDef=savedWidgetDefs[i];

          // widget definition to use
          var widgetDefinition=this.widgetDefinitions.getByName(savedWidgetDef.name);

          // check for no widget
          if (!widgetDefinition){
            // no widget definition found, remove and return false
            $log.warn('Widget with name "' + savedWidgetDef.name + '" was not found in given widget definition objects');
            continue;
          }

          // check widget-specific storageHash
          if (widgetDefinition.hasOwnProperty('storageHash') && widgetDefinition.storageHash !==savedWidgetDef.storageHash){
            // widget definition was found, but storageHash was stale, removing storage
            $log.info('Widget Definition Object with name "' + savedWidgetDef.name + '" was found ' +
              'but the storageHash property on the widget definition is different from that on the ' +
              'serialized widget loaded from storage. hash from storage: "' + savedWidgetDef.storageHash + '"' +
              ', hash from WDO: "' + widgetDefinition.storageHash + '"');
            continue;
          }

          // push instantiated widget to result array
          result.push(savedWidgetDef);
        }

        return result;
      },

      _handleAsyncLoad: function(promise){
        var self=this;
        var deferred=$q.defer();
        promise.then(
          // success
          function(res){
            var result=self._handleSyncLoad(res);
            if (result){
              deferred.resolve(result);
            } else{
              deferred.reject(result);
            }
          },
          // failure
          function(res){
            deferred.reject(res);
          }
        );

        return deferred.promise;
      }

    };
    return DashboardState;
  }]);

/*
 * Define widget data model factory.
 */
'use strict';
angular.module('ui.dashboard').factory('WidgetDataModel', function (){
    function WidgetDataModel(){
    }

    WidgetDataModel.prototype={
      setup: function (widget, scope){
        this.dataAttrName=widget.dataAttrName;
        this.dataModelOptions=widget.dataModelOptions;
        this.widgetScope=scope;
      },

      updateScope: function (data){
        this.widgetScope.widgetData=data;
      },

      init: function (){
        // to be overridden by subclasses
      },

      destroy: function (){
        // to be overridden by subclasses
      }
    };

    return WidgetDataModel;
});

/*
 * Widget defination collection factory.
 */
'use strict';
angular.module('ui.dashboard').factory('WidgetDefCollection', function (){
    function WidgetDefCollection(widgetDefs){
      this.push.apply(this, widgetDefs);

      // build (name -> widget definition) map for widget lookup by name
      var map={};
      _.each(widgetDefs, function (widgetDef){
        map[widgetDef.name]=widgetDef;
      });
      this.map=map;
    }

    WidgetDefCollection.prototype=Object.create(Array.prototype);

    WidgetDefCollection.prototype.getByName=function (name){
      return this.map[name];
    };

    return WidgetDefCollection;
});

/*
 * Define widget model factory.
 */
'use strict';
angular.module('ui.dashboard')
.factory('WidgetModel', function (){
    // constructor for widget model instances
    function WidgetModel(Class, overrides){
      var defaults={
          title: 'Widget',
          name: Class.name,
          attrs: Class.attrs,
          dataAttrName: Class.dataAttrName,
          dataModelType: Class.dataModelType,
          dataModelArgs: Class.dataModelArgs, // used in data model constructor, not serialized
          //AW Need deep copy of options to support widget options editing
          dataModelOptions: Class.dataModelOptions,
          settingsModalOptions: Class.settingsModalOptions,
          onSettingsClose: Class.onSettingsClose,
          onSettingsDismiss: Class.onSettingsDismiss,
          style: Class.style ||{},
          size: Class.size ||{}
        };

      overrides=overrides ||{};
      angular.extend(this, angular.copy(defaults), overrides);
      this.containerStyle={ width: '33%' }; // default width
      this.contentStyle={};
      this.updateContainerStyle(this.style);

      if (Class.templateUrl){
        this.templateUrl=Class.templateUrl;
      } else if (Class.template){
        this.template=Class.template;
      } else{
        var directive=Class.directive || Class.name;
        this.directive=directive;
      }

      if (this.size && _.has(this.size, 'height')){
        this.setHeight(this.size.height);
      }

      if (this.style && _.has(this.style, 'width')){ //TODO deprecate style attribute
        this.setWidth(this.style.width);
      }

      if (this.size && _.has(this.size, 'width')){
        this.setWidth(this.size.width);
      }
    }

    WidgetModel.prototype={
      // sets the width (and widthUnits)
      setWidth: function (width, units){
        width=width.toString();
        units=units || width.replace(/^[-\.\d]+/, '') || '%';
        this.widthUnits=units;
        width=parseFloat(width);

        if (width < 0){
          return false;
        }

        if (units==='%'){
          width=Math.min(100, width);
          width=Math.max(0, width);
        }

        this.containerStyle.width=width + '' + units;
        this.updateSize(this.containerStyle);
        return true;
      },

      setHeight: function (height){
        this.contentStyle.height=height;
        this.updateSize(this.contentStyle);
      },

      setStyle: function (style){
        this.style=style;
        this.updateContainerStyle(style);
      },

      updateSize: function (size){
        angular.extend(this.size, size);
      },

      updateContainerStyle: function (style){
        angular.extend(this.containerStyle, style);
      }
    };

    return WidgetModel;
});

/*
 * Dashboard Widget controller.
 */
'use strict';
angular.module('ui.dashboard')
    .controller('DashboardWidgetCtrl', ['$scope', '$element', '$compile', '$window', '$timeout',
    function($scope, $element, $compile, $window, $timeout){

      $scope.status={
        isopen: false
      };

      /*Fills "container" with compiled view*/
      $scope.makeTemplateString=function(){
        var widget=$scope.widget;

        // First, build template string
        var templateString='';

        if (widget.templateUrl){

          // Use ng-include for templateUrl
          templateString='<div ng-include="\'' + widget.templateUrl + '\'"></div>';

        } else if (widget.template){

          // Direct string template
          templateString=widget.template;

        } else{

          // Assume attribute directive
          templateString='<div ' + widget.directive;

          // Check if data attribute was specified
          if (widget.dataAttrName){
            widget.attrs=widget.attrs ||{};
            widget.attrs[widget.dataAttrName]='widgetData';
          }

          // Check for specified attributes
          if (widget.attrs){

            // First check directive name attr
            if (widget.attrs[widget.directive]){
              templateString +='="' + widget.attrs[widget.directive] + '"';
            }

            // Add attributes
            _.each(widget.attrs, function(value, attr){

              // make sure we aren't reusing directive attr
              if (attr !==widget.directive){
                templateString +=' ' + attr + '="' + value + '"';
              }

            });
          }
          templateString +='></div>';
        }
        return templateString;
      };

    /* Grab resizer function for width. */
    $scope.grabResizer=function(e){
        var widget=$scope.widget;
        var widgetElm=$element.find('.widget_new');

        // ignore middle- and right-click
        if (e.which !==1){
          return;
        }

        e.stopPropagation();
        e.originalEvent.preventDefault();

        // get the starting horizontal position
        var initX=e.clientX;

        // Get the current width of the widget and dashboard
        var pixelWidth=widgetElm.width();
        var pixelHeight=widgetElm.height();
        var widgetStyleWidth=widget.containerStyle.width;
        var widthUnits=widget.widthUnits;
        var unitWidth=parseFloat(widgetStyleWidth);

        // create marquee element for resize action
        var $marquee=angular.element('<div class="widget-resizer-marquee" style="height: ' + pixelHeight + 'px; width: ' + pixelWidth + 'px;"></div>');
        widgetElm.append($marquee);

        // determine the unit/pixel ratio
        var transformMultiplier=unitWidth / pixelWidth;

        // updates marquee with preview of new width
        var mousemove=function(e){
          var curX=e.clientX;
          var pixelChange=curX - initX;
          var newWidth=pixelWidth + pixelChange;
          $marquee.css('width', newWidth + 'px');
        };

        // sets new widget width on mouseup
        var mouseup=function(e){
          // remove listener and marquee
          jQuery($window).off('mousemove', mousemove);
          $marquee.remove();

          // calculate change in units
          var curX=e.clientX;
          var pixelChange=curX - initX;
          var unitChange=Math.round(pixelChange * transformMultiplier * 100) / 100;

          // add to initial unit width
          var newWidth=unitWidth * 1 + unitChange;
          widget.setWidth(newWidth + widthUnits);
          $scope.$emit('widgetChanged', widget);
          $scope.$apply();
          $scope.$broadcast('widgetResized',{
            width: newWidth
          });
        };

        jQuery($window).on('mousemove', mousemove).one('mouseup', mouseup);
  };

    /* Grab Resizer function for height. */
    $scope.grabSouthResizer=function(e){
        var widgetElm=$element.find('.widget_new');

        // ignore middle- and right-click
        if (e.which !==1){
          return;
        }

        e.stopPropagation();
        e.originalEvent.preventDefault();

        // get the starting horizontal position
        var initY=e.clientY;

        // Get the current width of the widget and dashboard
        var pixelWidth=widgetElm.width();
        var pixelHeight=widgetElm.height();

        // create marquee element for resize action
        var $marquee=angular.element('<div class="widget-resizer-marquee" style="height: ' + pixelHeight + 'px; width: ' + pixelWidth + 'px;"></div>');
        widgetElm.append($marquee);

        // updates marquee with preview of new height
        var mousemove=function(e){
          var curY=e.clientY;
          var pixelChange=curY - initY;
          var newHeight=pixelHeight + pixelChange;
          $marquee.css('height', newHeight + 'px');
        };

        // sets new widget width on mouseup
        var mouseup=function(e){
          // remove listener and marquee
          jQuery($window).off('mousemove', mousemove);
          $marquee.remove();

          // calculate height change
          var curY=e.clientY;
          var pixelChange=curY - initY;

          var widgetContainer=widgetElm.find('.dashboard-widget-content');

          var diff=pixelChange;
          var height=parseInt(widgetContainer.css('height'), 10);
          var newHeight=(height + diff);

          $scope.widget.setHeight(newHeight + 'px');

          $scope.$emit('widgetChanged', $scope.widget);
          $scope.$apply(); // make AngularJS to apply style changes

          $scope.$broadcast('widgetResized',{
            height: newHeight
          });
        };

        jQuery($window).on('mousemove', mousemove).one('mouseup', mouseup);
    };

    /* Compile template function */
    $scope.compileTemplate=function(){
        var container=$scope.findWidgetContainer($element);
        var templateString=$scope.makeTemplateString();
        var widgetElement=angular.element(templateString);

        container.empty();
        container.append(widgetElement);
        $compile(widgetElement)($scope);
    };

    /* Widget container find function. */
    $scope.findWidgetContainer=function(element){
        // widget placeholder is the first (and only) child of .dashboard-widget-content
        return element.find('.dashboard-widget-content');
    };
 } 
]);

/* filter function to remove space from string and convert in lower case letter */
angular.module("ui.dashboard").filter('removeSpacesThenLowercaseFilter', function (){
    return function (text){
        var str=text.replace(/\s+/g, '');
        return str.toLowerCase();
    };
});

angular.module("ui.dashboard").run(["$templateCache", function($templateCache){

    /*Dashboard widget common template*/  /*ui-sortable=\"sortableOptions\" */
    /*L1037	"<span class=\"span-color highlight-identifier\" style=\" margin-left: 11px;float: right;\" title=\"Click to Drag\"><i class=\"icon-              move\"></i></span>\n" +  */
    $templateCache.put("template/dashboard.html",
    "<div>\n" +
    /*"    <div ng-model=\"widgets\" class=\"dashboard-widget-area\">\n" +*/
    "        <div id=\"toruid{{widget.name}}\" ng-repeat=\"widget in widgets\" ng-style=\"widget.containerStyle\" class=\"widget-container\" widget>\n" +
    "            <div class=\"widget_new panel panel-default\" >\n" +  
    "                <div class=\"widget-header panel-heading\">\n" +
    "                    <h3 class=\"panel-title\">\n" +
    "                        <span class=\"label label-primary heading-font-style \" ng-if=\"!options.hideWidgetName\">{{widget.name}}</span>\n" +
    "                        <span hidden =true class=\"span-color highlight-identifier\" style=\" margin-left: 11px;float: right;\" title=\"Click to Drag\"><i class=\"icon-move\"></i></span>\n" +   
    "                        <span ng-click=\"removeWidget(widget);\" class=\"closewidget highlight-identifier span-color\" ng-if=\"!options.hideWidgetClose\"></span>\n" +     
    "                        <span ng-click=\"openWidgetSettings(widget, $index);\" title=\"Click to configure\" class=\"configwidget btn-clear icon-gearalt highlight-identifier span-color\" ng-if=\"!options.hideWidgetSettings\"></span>\n" +
	"                        <button ng-click=\"openManualRefreshSettings(widget , $index);\" class=\"btn-clear timer-refresh \" id='ID{{widget.name}}'  title=\"Click to refresh\" style=\"float: right;\"><i class=\"icon-refreshalt text-info\"></i></button>\n" +
    "                    </h3>\n" +
    "                </div>\n" +
    "                <div class=\"panel-body dashboard-widget-content\" ng-style=\"widget.contentStyle\"></div>\n" +
    "                <div ng-if=\"options.enableHorizontalResize\" class=\"widget-ew-resizer\" ng-mousedown=\"grabResizer($event)\"></div>\n" +
    "                <div ng-if=\"options.enableVerticalResize\" class=\"widget-s-resizer\" ng-mousedown=\"grabSouthResizer($event)\"></div>\n" +
    "            </div>\n" +
    "        </div>\n" +
    /*"    </div>\n" +*/
    "</div>"
    );
}]);