(function($, PHD, window, undefined) {
    
    /*
    *   DataTable provides behavior for building, updating and interacting with
    *   tables
    */

    var defaults = {
        select: true,
        scrollSelect: true, // new option to allow non-scrolling of selected item to top.
        multiselect: false,
        className: "",
        prefix: "",
        stateKeys: null,    // sub-class based on subset of data (was state)
        cellClasses: null,
        sortable: false,
        sortableCells: null,
        scroll: false,
        filter: false,
        resize: false,
        paged: false,
        integerIDs: true,
        prependData: false,
        placeholder: gettext("There is no data to display"),
        loading: '<i class="icon-restart icon-spin"></i>'
    };

    /* 
    *   Object to convert raw values to display-friendly values
    *   Contains a few common formatters — the rest are provided by
    *   the view that instantiates the DataTable
    */
    var defaultFormatter = {
        getValue: function(key, value, row) {
            if ((typeof value === 'undefined') || value === null)
                value = EMPTY_CELL_TEXT;
            return this.hasOwnProperty(key) ? 
                this[key](value, row, key) :
                value;
        },
        name: function(value, row) {
            return '<span class="truncates item-name" title="' + value + '">' + value + '</span>';
        },
        percent_complete: function(value, row) {
            return '<div class="progress-bar" title="' + value + '%">' +
                '<div class="progress-meter" style="width:' + value + '%;"></div></div><span class="progress-count">'+ value + '%</span>';
        },
        message: function(value) {
            return '<span class="truncates">' + value + '</span>';
        }
    };

    var lookup = PHD.StateLookup;

    var DataTable = function DataTable($table, options, data) {
        if(this instanceof DataTable) {
            this.$table = $table;
            this.$thead = this.$table.children("thead");
            this.$tbody = this.$table.children("tbody");
            this.$tfoot = this.$table.children("tfoot");

            this.options = $.extend({}, defaults, options);

            this.data = data || null;
            this.cols = 0;
            this.rows = {};
            this.dbids = [];

            this.$selected = $();
            this.selectedID = null;

            this._insert = Array.prototype.push;
            this._isLoading = false;

            this.formatter = $.extend({}, defaultFormatter);
        } else {
            return new DataTable($table, options, data).init();
        }
    };

    DataTable.prototype = {
        init: function() {
            var self = this,
                $cols = this.$thead.find("tr:last-child th"),
                cols = $cols.length,
                $phtd = $("<td>", {"colspan": cols}).html(this.options.placeholder),
                $phtr = $("<tr>", {"class": "placeholder"}).append($phtd),
                $ldtd = $("<td>", {"colspan": cols}).html(this.options.loading),
                $ldtr = $("<tr>", {"class": "loading-row"}).append($ldtd);

            if(this.options.select) {
                this.options.className = "clickable " + this.options.className;

                // Keyboard controls
                $("body").on("keydown", function(event) {
                    switch ( event.keyCode ) {
                        case $.ui.keyCode.UP:
                            event.preventDefault();
                            if(self.hasSelectedRow()) {
                                self.selectPreviousRow();
                            }
                            break;
                        case $.ui.keyCode.DOWN:
                            event.preventDefault();
                            if(self.hasSelectedRow()) {
                                self.selectNextRow();
                            }
                            break;
                    }
                });
            }

            if(this.options.prependData) {
                this._insert = Array.prototype.unshift;
            }

            if (this.options.prefix) {
                this.prefix = this.options.prefix;
            }

            this.schema = $cols.map(this.dataSource).get();

            this.cols = cols;
            
            this.$placeholder = this.$tbody.find(".placeholder");
            if(!this.$placeholder.length) {
                this.$placeholder = $phtr;
            }
            this.$table.data("placeholder", this.$placeholder);

            this.$loading = this.$tbody.find(".loading-row");
            if(!this.$loading.length) {
                this.$loading = $ldtr;
            }

            //this.$tbody.append(this.$placeholder);

            this.$tbody.on("click", ".clickable", $.proxy(this.handleClick, this));

            if(this.data)
                return this.load(this.data);

            return this.adopt();
        },

        adopt: function() {
            var $rows = this.$tbody.find("tr:not(.placeholder)");
            return this;
        },

        empty: function() {
            this.$tbody.find("tr:not(.placeholder)").remove();
            this.rows = {};
            this.dbids = [];

            this.$selected = $();
            this.selectedID = null;
            return this;
        },

        showPlaceholder: function() {
            this.$placeholder.prependTo(this.$tbody);
            return this;
        },

        hidePlaceholder: function() {
            this.$placeholder.detach();
            return this;
        },

        showLoadingIndicator: function() {
            this._isLoading = true;
            this.empty();
            if(this.$placeholder.is(":visible")) this.hidePlaceholder();
            this.$loading.prependTo(this.$tbody);
            return this;
        },

        hideLoadingIndicator: function() {
            this._isLoading = false;
            this.$loading.detach();
            return this;
        },

        getIDs: function(data) {
            return _.pluck(data, "id");
        },

        load: function(data) {
            this.empty();
            this.data = data;
            this.dbids = this.getIDs(this.data);
            return this.build();
        },

        insertRows: function(chunk, start) {
            var self = this,
                offset = start || 0,
                rows,
                html = "",
                $rows = $();

            if(chunk) {
                rows = this.dbids.slice(offset, offset+50);
            } else {
                rows = this.dbids.slice(0);
            }

            rows.forEach(function(rowid) {
                html += self.rows[rowid].$el[0].outerHTML;
            });
            this.$tbody.html(html);
        },

        build: function() {
            var self = this;
	        var dataLength = 0;

            if (typeof this.dbids !== 'undefined') {
                chunk = (this.dbids.length > 50);
            } else {
                chunk = false;
            }

	        // handle no data
	        if (typeof this.data !== 'undefined' && this.data != null && this.data.length) {
		        dataLength = this.data.length;
                this.hidePlaceholder();
                this.data.forEach(function(row) {
                    var dbid = row.id;
                    self.rows[dbid] = DataRow(self, row, self.options.className, self.options.cellClasses);
                    self.rows[dbid].$el.appendTo(self.$tbody);
                    self.$table.trigger("rowadd", [self.rows[dbid], dbid, self]);
                });
            }

	         if (!dataLength) {
                this.showPlaceholder();
            }

            this.$table.trigger("loadfinished", [(dataLength > 0), this]);
            return this;
        },

        update: function(data, deleteMissing) {
            if(typeof this.data === 'undefined' || this.data === null) return this.load(data);

            var self = this,
                newRows = false,
                deleteMissing = (deleteMissing === true);

            if(deleteMissing) {
                var newIDs = this.getIDs(data),
                    toDelete = _.difference(this.dbids, newIDs);
                this.deleteRows(toDelete);
            }

            this.data = data;
            if(typeof data !== 'undefined' && data.length) {
                this.hidePlaceholder();
                this.data.forEach(function(row) {
                    var dbid = row.id;
                    if(_.contains(self.dbids, dbid) && (typeof self.rows[dbid] !== 'undefined')) {
                        self.rows[dbid].update(row);
                        self.$table.trigger("rowupdate", [self.rows[dbid], dbid, self]);
                    } else {
                        newRows = true;
                        self._insert.call(self.dbids, dbid);
                        self.rows[dbid] = DataRow(self, row, self.options.className, self.options.cellClasses);
                        self.rows[dbid].$el.appendTo(self.$tbody);
                        self.$table.trigger("rowadd", [self.rows[dbid], dbid, self]);
                    }
                });
            } else {
                if(deleteMissing) {
                    this.empty();
                    this.showPlaceholder();
                }
            }

            if(this.selectedID) {
                this.selectByID(this.selectedID);
            }

            this.$table.trigger("updatefinished", [newRows, this]);
            return this;
        },

        handleClick: function(event) {
            //event.stopPropagation();
            if(!this._isLoading) this.select($(event.currentTarget));
        },

        selectByID: function(dbid) {
            if(this.rows.hasOwnProperty(dbid)) {
                return this.select(this.rows[dbid].$el);
            } else {
                return false;
            }
        },

        select: function($row, silent) {
            if($row.is(".current-row")) {
                return false;
            }
            if(this.$selected.length) {
                this.$tbody.find(".current-row").removeClass("current-row");
            }
            this.$selected = $row.addClass("current-row").focus();
            if(this.options.integerIDs) {
                this.selectedID = parseInt(this.$selected.data("dbid"), 10);    
            } else {
                this.selectedID = this.$selected.data("dbid");
            }
            if (this.options.scrollSelect) {
                //console.log("scrollSelect: OffsetTop = " + this.$selected[0].offsetTop + ", and selected height = " + this.$selected.height());
                this.$tbody.scrollTop(this.$selected[0].offsetTop - this.$selected.height());
            }

            this.$selected
                .trigger("rowselect", [this.selectedID, $row, this, this.getDataRow(this.selectedID)]);

            return this;
        },

        selectNextRow: function() {
            var next = this.$selected.next("tr");
            return next.length ? this.select(next) : false;
        },

        selectPreviousRow: function() {
            var prev = this.$selected.prev("tr");
            return prev.length ? this.select(prev) : false;
        },

        deselect: function() {
            this.$selected.removeClass("current-row");
            this.$selected = $();
            this.selectedID = null;
            return this;
        },

        hasSelectedRow: function() {
            return (this.selectedID !== null && this.$selected.length);
        },

        hasRow: function(dbid) {
            return this.rows.hasOwnProperty(dbid);
        },

        getRow: function(dbid) {
            return this.rows[dbid].$el;
        },

        getDataRow: function(dbid) {
            return this.rows[dbid];
        },

        getRows: function() {
            return this.$tbody.find("tr:not(.placeholder)");
        },

        getSelectedRow: function() {
            if(this.hasSelectedRow()) {
                return this.$selected;
            }
        },

        count: function() {
            return this.dbids.length;
        },

        deleteRows: function(rows) {
            rows.forEach($.proxy(this.deleteRow, this));
            if(!this.dbids.length) {
                this.showPlaceholder();
            }
            return this;
        },

        deleteRow: function(dbid) {
            if(dbid === this.selectedID) {
                this.selectedID = null;
                this.$selected = $();
            }
            if(this.rows.hasOwnProperty(dbid)) {
                this.rows[dbid].$el.remove();
                delete this.rows[dbid];
                this.dbids = _.without(this.dbids, dbid);
                this.$table.trigger("rowdelete", [dbid, this]);
            }
            return this;
        },

        dataSource: function() {
            return $(this).data("source");
        },

        registerFormatters: function(formatters) {
            $.extend(this.formatter, formatters);
            return this;
        },

        getData: function(dbid, key) {
            dbid = parseInt(dbid, 10);
            if(this.hasRow(dbid)) {
                return this.rows[dbid].data[key];
            } else {
                return "Row Does Not Exist";
            }

        },

        on: function(eventName, callback) {
            this.$table.on(eventName, callback);
            return this;
        },

        off: function(eventName) {
            this.$table.off(eventName);
            return this;
        }
    };

    PHD.DataTable = DataTable;

    var DataRow = function DataRow(table, data, rowClass, cellClasses) {
        if(this instanceof DataRow) {
            this.Parent = table;
            this.schema = this.Parent.schema;
            this.data = data;
            this.$el = $();
            this.rowClass = rowClass;
            this.cellClasses = cellClasses;
        } else {
            return new DataRow(table, data, rowClass, cellClasses).init();
        }
    };

    DataRow.prototype = {
        init: function() {
            return this.tr();
        },
        tr: function() {
            var self = this,
                tds = this.Parent.schema.map($.proxy(this.td, this)).join(""),
                //lmc - remove state
                //$row = $("<tr>", {"data-dbid":self.data.state.id, "class": this.rowClass}).html(tds);
                $row = $("<tr>", {"data-dbid":self.data.id, "class": this.rowClass}).html(tds);

            //this.$el = $row.classify(this.data.state, lookup);
		    // lmc - remove state BUT TAKES LONGER!!
            if (this.Parent.options.prefix) {
		        prefix = this.Parent.options.prefix;
	        } else {
		        prefix = null;
	        }
            this.$el = $row.classify(this.data, lookup, prefix);
            self.enrich();
            return this;
        },
        td: function(key, index, schema) {
            var value = this.data[key],
                cell = '<td data-source="' + key + '"';

            if(this.cellClasses && this.cellClasses[index]) {
                cell += ' class="' + this.cellClasses[index] + '"';
            }
            
            if(this.Parent.options.sortable) {
                if(this.Parent.options.sortableCells === null || this.Parent.options.sortableCells[index]) {
                    cell += ' data-sort-value="' + this.getSortValue(key) + '"';
                }
            }

            cell += '>';
            return cell + this.format(key, value, this.data) + '</td>';
        },
        update: function(data) {
            var self = this;

            this.$el.find("td").each(function() {
                var $cell = $(this),
                    key = $cell.data("source");

                if(key && key !== "controls") {
                    //if(data.data.hasOwnProperty(key)) {
                    //    if(data.data[key] !== self.data.data[key]) {
                    //       $cell.html(self.format(key, data.data[key], data));
                    if(data.hasOwnProperty(key)) {
                        if(data[key] !== self.data[key]) {
                           $cell.html(self.format(key, data[key], data));
                        }
                    } else {
                        $cell.html(self.format(key, null, data));
                    }
                }
                if($cell.hasAttr('data-sort-value')) {
                    $cell.data("sort-value", self.getSortValue(key, data));
                }
                
            });

            this.data = data;

            //self.$el.classify(this.data.state, lookup);
            // lmc remove state NEED TO subclass
	        self.$el.classify(this.data, lookup, prefix);
            self.enrich();

            return this;
        },
        getSortValue: function(key, data) {
            data = data || this.data;
            return (data.sort && data.sort.hasOwnProperty(key)) ? data.sort[key] : data[key];
        },
        format: function(key, value, data) {
            return this.Parent.formatter.getValue(key, value, data || this.data);
        },
        enrich: function() {
            this.$el.find(".progress-bar, .table-icon").tooltip({
                position: {
                    my: "center bottom",
                    at: "center top"
                },
                tooltipClass: "progress-bar-tooltip"
            });
        }
    };

    PHD.DataRow = DataRow;

    $.fn.classify = function(keys, lookup, prefix) {
        return this.each(function() {
            var $el = $(this);
            $.each(keys, function(key, value) {
                if(key === "id") { return; }
                var from_class = prefix != null ? prefix + key : key;
                $el.alterClass(from_class.replace("_", "-") + "-*", lookup.get(key, value, prefix));
            });
        });
    };

})(jQuery, window.PHD || (window.PHD = {}), window);
